@@ -1,22 +1,29 @@
 
 #' Add background OSM tiles
 #'
-#' Uses \link[rosm]{osm.image} to add background tiles.
+#' Uses [rosm::osm.image()] to add background tiles. If you are publishing
+#' a map using these tiles, make sure to use the proper attribution
+#' (e.g., "Copyright OpenStreetMap contributors" when using an
+#' OpenStreetMap-based tile set).
 #'
 #' @param type The map type
 #' @param zoom The zoom level (overrides zoomin)
-#' @param zoomin Delta on default zoom
+#' @param zoomin Delta on default zoom. The default value is designed
+#'   to download fewer tiles than you probably want. Use `-1` or `0` to
+#'   increase the resolution.
 #' @param forcedownload Re-download cached tiles?
 #' @param cachedir Specify cache directory
-#' @param progress Use progress = "none" to suppress progress and zoom output
-#' @param quiet Use quiet = FALSE to see which URLs are downloaded
-#' @param interpolate Parameter for raster
+#' @param progress Use `progress = "none"` to suppress progress and zoom output
+#' @param quiet Use `quiet = FALSE` to see which URLs are downloaded
+#' @param interpolate Passed to [grid::rasterGrob()]
+#' @param alpha Use to make this layer semi-transparent
 #' @param data,mapping Specify data and mapping to use this geom with facets
 #'
 #' @return A ggplot2 layer
 #' @export
 #'
 #' @examples
+#' library(ggplot2)
 #' load_longlake_data()
 #'
 #' ggplot() +
@@ -26,7 +33,7 @@
 annotation_map_tile <- function(type = "osm", zoom = NULL, zoomin = -2,
                                 forcedownload = FALSE, cachedir = NULL,
                                 progress = c("text", "none"), quiet = TRUE,
-                                interpolate = TRUE, data = NULL, mapping = NULL) {
+                                interpolate = TRUE, data = NULL, mapping = NULL, alpha = 1) {
 
   progress <- match.arg(progress)
   if(!is.null(zoom)) {
@@ -55,7 +62,8 @@ annotation_map_tile <- function(type = "osm", zoom = NULL, zoomin = -2,
         cachedir = cachedir,
         progress = progress,
         quiet = quiet,
-        interpolate = interpolate
+        interpolate = interpolate,
+        alpha = alpha
       ),
       inherit.aes = FALSE,
       show.legend = FALSE
@@ -91,7 +99,7 @@ GeomMapTile <- ggplot2::ggproto(
   draw_panel = function(
     data, panel_params, coordinates,
     forcedownload = FALSE, cachedir = NULL,
-    progress = c("none", "text"), quiet = TRUE, interpolate = TRUE
+    progress = c("none", "text"), quiet = TRUE, interpolate = TRUE, alpha = 1
   ) {
     progress <- match.arg(progress)
 
@@ -117,7 +125,7 @@ GeomMapTile <- ggplot2::ggproto(
       # because this involves a call to projectRaster(), it has to be wrapped in
       # suppressWarnings to avaoid the "no non-missing arguments to min; returning Inf" error
       raster <- suppressWarnings(
-          rosm::osm.raster(
+        rosm_raster(
           x = sp_bbox,
           zoomin = data[["zoomin"]][1],
           zoom = data[["zoom"]][1],
@@ -141,12 +149,12 @@ GeomMapTile <- ggplot2::ggproto(
 
       ext <- raster::extent(raster_proj)
       corners <- data.frame(x = c(ext@xmin, ext@xmax), y = c(ext@ymin, ext@ymax))
-      img <- raster_as_array(raster_proj)
+      img <- raster_as_array(raster_proj, alpha = alpha)
 
     } else {
 
       # can use osm.image, which is much faster (and this is the most common case)
-      img <- rosm::osm.image(
+      img <- rosm_image(
         x = sp_bbox,
         zoomin = data[["zoomin"]][1],
         zoom = data[["zoom"]][1],
@@ -159,6 +167,22 @@ GeomMapTile <- ggplot2::ggproto(
 
       bbox_img <- attr(img, "bbox")
       corners <- data.frame(t(bbox_img))
+
+      # apply the specified alpha
+      if(alpha != 1) {
+        # it could be that there is already a band 4, or it may not exist yet
+        if(dim(img)[3] == 4) {
+          tband <- img[ , , 4, drop = FALSE]
+        } else {
+          tband <- array(1, dim(img)[1:2])
+        }
+
+        # multiply the alpha band by the requested alpha (clamping to 0,1)
+        tband <- tband * max(0, min(1, alpha))
+
+        # bind it to the original raster
+        img <- abind::abind(img[, , 1:3, drop = FALSE], tband)
+      }
     }
 
     # transform corners to viewport cs
@@ -176,3 +200,12 @@ GeomMapTile <- ggplot2::ggproto(
   }
 )
 
+# the CMD check doesn't detect the call to
+# rosm in the ggproto object and complains
+rosm_image <- function(...) {
+  rosm::osm.image(...)
+}
+
+rosm_raster <- function(...) {
+  rosm::osm.raster(...)
+}
